module ContainerTasks
  extend ActiveSupport::Concern
  module ClassMethods
  end

  included do
    desc 'lsc [KEYWORD] [-n nk] [-i ik]', <<-LONG
    'list all container which container name match the patten'
    example:
      dk lsc                    #list all containers
      dk lsc test               #list all containers which include string test;
      dk lsc test -n nk         #list all containers which include test and name include nk;
      dk lsc test -i ik         #list all containers which include test and image include ik;
      dk lsc -n nk -i tk        #list all containers which name include nk and image include ik
    LONG
    method_option :name, type: :string, aliases: 'n'
    method_option :image, type: :string, aliases: 'i'
    def lsc(keyword = nil)
      ret = Container.find(keyword: keyword, name: options['name'], image: options['image'])
      Container.print_table(ret)
    end

    desc 'del [KEYWORD] [-n nk] [-i ik]', <<-LONG
    'delete all containers which container data match the patten'
    example:
      dk del                    #delete all containers
      dk del test               #delete all containers which include string test;
      dk del test -n nk         #delete all containers which include test and name include nk;
      dk del test -i ik         #delete all containers which include test and image include ik;
      dk del -n nk -i tk        #delete all containers which name include nk and image include ik
    LONG
    method_option :name, type: :string, aliases: 'n'
    method_option :image, type: :string, aliases: 'i'
    def del(keyword = nil)
      ret = Container.find(keyword: keyword, name: options['name'], image: options['image'])
      Container.print_table(ret)
      say('The listed containers will be deleted, ARE YOU SURE.'.blue)
      answer = ask('(y=YES;q=QUIT;i=ASK one by one)', limited_to: %w[y q i])
      return if answer == 'q'
      ret.each do |container|
        if answer == 'i'
          say(container.to_s)
          each_answer = ask('going to delete? (y=YES,n=NO,q=QUIT):')
          next if each_answer == 'n'
          break if each_answer == 'q'
        end
        container.del
      end
    end
    map rm: :del
    map rmc: :del

    desc 'exec [KEYWORD] [-n nk] [-i ik] [--opts] [--cmd] [-s]', <<-LONG
    select container which container_name match the patten and select some to run
    example:
      dk exec\
      #list all container and select some to exec

      dk exec test\
      #list all container which include string test and select one to exec;

      dk exec test -n nk\
      #list all container which include test and name include nk and select one to exec;

      dk exec test -t tk\
      #list all container which include test and tag include tk and select one to exec;

      dk exec -n nk -t tk\
      #list all container which repository include nk and tag include tk and select one to push;

      dk exec --opts "'-i -t --name test'" --cmd '/bin/bash'\
      #NOTE  opts and cmd option ,use "''" to quot as sample
      #list all container and exec as opts -it --name test and command is /bin/bash

      dk exec --opts "'-i -t'" --cmd '/bin/bash' -s test.bash\
      #NOTE  opts and cmd option ,use "''" to quot as sample
      #list all container and exec as opts -it --name test and command is /bin/bash
      #and save the command as alias and alias name is test.bash
    LONG
    method_option :opts, type: :string, aliases: 'o'
    method_option :cmd, type: :string, aliases: 'c'
    method_option :name, type: :string, aliases: 'n'
    method_option :image, type: :string, aliases: 'i'
    method_option :alias, type: :string, aliases: 's'

    def exec(keyword = nil)
      ret = Container.find(keyword: keyword, name: options['name'], image: options['image'])
      if ret.count > 1
        Container.print_table(ret) { |item, index| "[#{index + 1}]#{item}" }
        question = ['please select one container to run:'].unshift(' ').unshift(' ').join("\n")
        say(question.blue)
        answer = ask('(q=QUIT;n=)', limited_to: ['q', *('1'..ret.count.to_s).to_a])
        return if answer == 'q'
      end

      command = options['cmd'] || ask_with_default('Any command you want to run?', 'echo hello')
      opts = options['opts'] || ask_with_default('Any option you want to set?', '-it')
      index = (answer ? answer.to_i : 1) - 1
      xcommand = ret[index].exec(opts, command)
      run(['dk', 'alias', options['alias'], xcommand].join(' ')) if options['alias']
      run(xcommand.join(' '))
    end
  end
end
