module Base
  extend ActiveSupport::Concern
  module ClassMethods
    def raw_lines
      hammer.run(command, capture: true).lines
    end

    def load(lines = raw_lines)
      @headers = lines.first
      @items = lines[1..-1].map do |line|
        create(line)
      end
    end

    def all
      load
    end

    def headers_raw
      @headers
    end

    def search(*keywords)
      keyword, = keywords
      keyword ||= '.'
      regexp = /#{keyword}/i
      ret = load.reject do |item|
        item.regexp = regexp
        regexp.match(item.raw).nil?
      end
      ret
    end

    def print_table(data)
      hammer.say @headers
      ret = data.each_with_index.map do |item, index|
        item = yield(item, index) if block_given?
        item.to_s
      end.join('')
      hammer.say ret
    end

    def run(*cmds)
      hammer.run(cmds.join(' '))
    end
  end

  included do
    attr_accessor :raw, :regexp
    def to_s
      @raw.gsub!(@regexp, '\0'.yellow) if @regexp
      end

    def run(*cmds)
      hammer.run(cmds.join(' '))
    end
  end
end
