class Image
  include Base
  attr_accessor :id, :repository, :tag, :created, :size, :regexp_repo, :regexp_tag
  def self.headers
    {
      repository: 'REPOSITORY',
      tag: 'TAG',
      id: 'IMAGE ID',
      created: 'CREATED',
      size: 'SIZE'
    }
  end

  def self.command
    'docker images'
  end

  def self.launch(image_name, run_opts, command)
    ['docker', 'run', run_opts, image_name, command]
  end

  def self.build(dockerfile, *tags)
    run 'docker',
        'build',
        '--force-rm',
        '-f',
        dockerfile,
        '-t',
        tags.join(' -t '),
        '.'
  end

  def self.find(keyword:, name:, tag:)
    return all unless keyword || name || tag
    ret = keyword ? search(keyword) : all
    if name
      regexp_repo = /#{name}/i
      ret = ret.select do |image|
        image.regexp_repo = regexp_repo
        regexp_repo.match image.repository
      end
    end
    if tag
      regexp_tag = /#{tag}/i
      ret = ret.select do |image|
        image.regexp_tag = regexp_tag
        regexp_tag.match image.tag
      end
    end
    ret
  end

  def self.push(repo_name)
    run("docker push #{repo_name}")
  end

  def self.create(line)
    repository, tag, id, created, size = line.chomp.split(' ' * 2).reject(&:empty?).map(&:strip)
    ret = new(repository: repository, tag: tag, id: id, created: created, size: size, raw: line)
    ret
  end

  def initialize(**params)
    @repository = params[:repository]
    @tag = params[:tag]
    @id = params[:id]
    @created = params[:created]
    @size = params[:size]
    @raw = params[:raw]
  end

  def del
    instance = /<none>/.match(image_name) ? id : image_name
    run("docker rmi -f #{instance}")
  end

  def self.pull(name)
    run("docker pull #{name}")
  end

  def push
    self.class.push(image_name)
  end

  def self.tag(name, *as_names)
    run("docker tag #{name} #{as_names.join(' ')}")
  end

  def image_name
    "#{repository}:#{tag || 'latest'}"
  end

  def launch(run_opts, command)
    self.class.launch(image_name, run_opts, command)
  end

  def to_a(*cols)
    cols = Image.headers.keys if cols.empty?
    ret = cols.map { |col| send(col).strip }
    ret
  end

  def to_s
    @raw.gsub!(@regexp, &:yellow) if @regexp
    @raw.sub!(repository, repository.gsub(@regexp_repo, '\0'.red)) if @regexp_repo
    @raw.gsub!(/#{tag}$/) { |tag| tag.gsub(@regexp_tag, '\0'.blue) } if @regexp_tag
    @raw
  end
end
